""" 
Python implementation of the color map function for the PASCAL VOC data set. 
Official Matlab version can be found in the PASCAL VOC devkit 
http://host.robots.ox.ac.uk/pascal/VOC/voc2012/index.html#devkit
"""
import numpy as np
import matplotlib.pyplot as plt


def color_map(N=256, normalized=False):
    def bitget(byteval, idx):
        return (byteval & (1 << idx)) != 0

    dtype = "float32" if normalized else "uint8"
    cmap = np.zeros((N, 3), dtype=dtype)
    for i in range(N):
        r = g = b = 0
        c = i
        for j in range(8):
            r = r | (bitget(c, 0) << 7 - j)
            g = g | (bitget(c, 1) << 7 - j)
            b = b | (bitget(c, 2) << 7 - j)
            c = c >> 3

        cmap[i] = np.array([r, g, b])

    cmap = cmap / 255 if normalized else cmap
    return cmap


def color_map_viz(labels: list[str], background=0, void=255):
    nclasses = len(labels)
    if background is not None:
        labels.insert(background, "background")
        nclasses += 1
    if void is not None:
        labels.insert(void, "void")

    row_size = 50
    col_size = 500
    cmap = color_map()
    array = np.empty((row_size * (nclasses + 1), col_size, cmap.shape[1]), dtype=cmap.dtype)
    for i in range(nclasses):
        array[i * row_size : i * row_size + row_size, :] = cmap[i]
    array[nclasses * row_size : nclasses * row_size + row_size, :] = cmap[-1]

    plt.imshow(array)
    plt.yticks([row_size * i + row_size / 2 for i in range(len(labels))], labels)
    plt.xticks([])
    plt.show()



from geda.utils.files import download_file, unzip, path_exists
from geda.utils.pylogger import get_pylogger
from abc import abstractmethod
from pathlib import Path

log = get_pylogger(__name__)


class DataProvider:
    URL: str

    def __init__(self, urls: dict[str, str], root: str):
        self.urls = urls
        self.root = root
        Path(self.root).mkdir(parents=True, exist_ok=True)
        self.raw_root = f"{root}/raw"
        if not self.check_if_present():
            self.zip_filepaths = self.download()  # TODO
            self.unzip()
            self.split_ids = self._get_split_ids()
            self.arrange_files()
            self.create_labels()
            self.filepaths = self._get_filepaths()
        else:
            log.info(f"Dataset is already present")
            self.split_ids = self._get_split_ids()
            self.filepaths = self._get_filepaths()

    def download(self):
        zip_filepaths = []
        for name, url in self.urls.items():
            ext = url.split(".")[-1]
            zip_filepath = f"{self.root}/{name}.{ext}"
            if ext in ["zip", "tar", "gz"]:
                zip_filepaths.append(zip_filepath)
            if path_exists(zip_filepath):
                log.info(f"{zip_filepath} is already present. Download canceled")
            else:
                download_file(url, zip_filepath)
        return zip_filepaths

    def unzip(self, remove: bool = False):
        if path_exists(self.raw_root):
            log.info(f"{self.raw_root} is already present. Unzip canceled")
            return
        Path(self.raw_root).mkdir(parents=True, exist_ok=True)
        for zip_filepath in self.zip_filepaths:
            unzip(zip_filepath, self.root, remove)
        self.move_to_raw_root()
        if remove:
            self.zip_filepaths.clear()

    @abstractmethod
    def check_if_present(self) -> bool:
        raise NotImplementedError()

    @abstractmethod
    def move_to_raw_root(self):
        raise NotImplementedError()

    @abstractmethod
    def _get_split_ids(self):
        raise NotImplementedError()

    @abstractmethod
    def arrange_files(self):
        raise NotImplementedError()

    @abstractmethod
    def _get_filepaths(self):
        raise NotImplementedError()

    @abstractmethod
    def create_labels(self):
        raise NotImplementedError()



import logging
import colorlog

formatter = colorlog.ColoredFormatter(
    "%(asctime)s - %(log_color)s%(levelname)s%(reset)s - %(message)s",
    datefmt="%Y-%m-%d %H:%M:%S",
    log_colors={
        "DEBUG": "cyan",
        "INFO": "green",
        "WARNING": "yellow",
        "ERROR": "red",
        "CRITICAL": "bold_red",
    },
)

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)


def get_pylogger(name=__name__) -> logging.Logger:
    """Initializes multi-GPU-friendly python command line logger."""
    logger = logging.getLogger(name)
    logger.addHandler(stream_handler)
    logger.setLevel(logging.DEBUG)
    return logger



from geda.data_providers.base import DataProvider
from typing import Literal
from PIL import Image


class BaseSegmentationDataset:
    def __init__(
        self,
        root: str,
        split: Literal["train", "val", "test"],
        data_provider: DataProvider,
    ):
        self.split = split
        self.ids = data_provider.split_ids[split]
        self.images_paths = data_provider.filepaths["images"][split]
        self.masks_paths = data_provider.filepaths["masks"][split]
        self.root = root

    def __getitem__(self, idx):
        image_fpath = self.images_paths[idx]
        mask_fpath = self.masks_paths[idx]
        image = Image.open(image_fpath).convert("RGB")
        mask = Image.open(mask_fpath)
        return image, mask

    def __len__(self):
        return len(self.images_paths)



from geda.utils.colors import color_map
import png
import numpy as np


def save_gray_array_as_color_png(
    array: np.ndarray, n_colors: int = 256, filename: str = "gray2color.png"
):
    height, width = array.shape
    palette = color_map(n_colors)
    w = png.Writer(width, height, palette=palette, bitdepth=8)
    f = open(filename, "wb")
    w.write(f, array.tolist())
